//= require <dialogs/default.js>
//= require <util.js>

Ext.namespace('dSA', 'dSA.actions', 'dSA.util', 'dSA.dialogs',
              'dSA.actions.dsmfreqsettings');

dSA.actions.dsmfreqsettings.FrequencySettings = Ext.extend(Ext.Panel, {
    initComponent: function() {
        var dsuidrenderer = function(value, meta, record, row, column, store)
        {
            var dsuid = record.get('dsuid');
            return String.format('<div ext:qtip="{0}">{1}</div>', dsuid,
                                 dSA.util.shortDSUID(dsuid));
        };

        var columns =
        [
            {
                header: dSA._("dSUID"),
                sortable: true,
                dataIndex: 'dsuid',
                renderer: dsuidrenderer
            },
            {
                header: dSA._("Name"),
                sortable: true,
                dataIndex: 'name'
            },
            {
                header: dSA._("Frequency"),
                dataIndex: 'description'
            }
        ];

        var rec = Ext.data.Record.create(
        [
            'dsuid', 'name', 'updstreamfrequency', 'description'
        ]);
        var reader = new Ext.data.JsonReader({}, rec);
        Ext.apply(this,
        {
            scope: this,
            cls: 'x-panel-no-top-border',
            bodyCfg:
            {
                cls: 'x-panel-no-top-border'
            },
            bubbleEvents: [ 'datasaved' ],
            items:
            [   {
                    xtype: 'panel',
                    border: false,
                    html: dSA._("Select dSMs directly in the grid, " +
                                "multiple selections are allowed."),
                    bodyCfg:
                    {
                        cls: 'x-window-body x-window-mc '
                    },
                    height: 30
                },
                {
                    xtype: 'grid',
                    height: 150,
                    columns: columns,
                    ref: 'dsmgrid',
                    store: new Ext.data.Store(
                    {
                        reader: reader
                    }),
                    stripeRows: true,
                    enableDragDrop: false,
                    viewConfig: 
                    {
                        forceFit: true
                    },
                    autoscroll: true
                },
                {
                    xtype: 'panel',
                    padding: 5,
                    ref: 'cbpanel',
                    height: 60,
                    border: false,
                    disabled: true,
                    bodyCfg:
                    {
                        cls: 'x-window-body x-window-mc '
                    },
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSA._("New upstream frequency for selected " +
                                        "dSMs") + ':'
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'combo',
                            ref: 'frequency',
                            name: 'frequency',
                            typeAhead: false,
                            lazyRender: true,
                            mode: 'local',
                            forceSelection: true,
                            editable: false,
                            triggerAction: 'all',
                            store: new Ext.data.ArrayStore({
                              id: 0,
                                fields:
                                [
                                    'id',
                                    'displayText'
                                ],
                                data:
                                [
                                    ["15/20", dSA._("15 kHz / 20 kHz")],
                                    ["12/20", dSA._("12 kHz / 20 kHz")],
                                    ["10/15", dSA._("10 kHz / 15 kHz")],
                                    ["10/12", dSA._("10 kHz / 12 kHz")]
                                ]
                            }),
                            valueField: 'id',
                            displayField: 'displayText',
                            value: "15/20"
                        }
                    ]
                }
            ]
        });

        dSA.actions.dsmfreqsettings.FrequencySettings.superclass.initComponent.apply(
                this, arguments);

        this.on('afterlayout',
            function()
            {
                this.el.mask(dSA._("Loading") + '...',
                                     'x-mask-loading');
                dSA.functions.LoadStructure(
                    this.LoadStructureCallback.createDelegate(this));
            },
            this, { buffer: 1000 });
        this.on('savepressed',
            function()
            {
                var frequency = this.cbpanel.frequency.getValue();

                var sel = this.dsmgrid.getSelectionModel();
                var rec = sel.getSelections();
                if (rec.length > 0)
                {
                    this.el.mask(dSA._("Saving") + '...', 'x-mask-loading');
                    var dsm = rec.shift();
                    dSA.functions.SavedSMUpstreamFrequency(dsm.get('dsuid'),
                        frequency, this.FreqSaveCallback.createDelegate(this,
                                                    [ frequency, rec ], true));
                }
            },
            this);
        
        this.dsmgrid.getStore().on('load', function(store, records, options)
        {
            this.el.unmask(); 
        },
        this);

        this.dsmgrid.on('rowclick', function(grid)
        {

            var sel = grid.getSelectionModel();
            var rec = sel.getSelected();
            if (rec)
            {
                this.cbpanel.enable();
            }
            else
            {
                this.cbpanel.disable();
            }
        },
        this);

    },
    LoadStructureCallback: function(result, data)
    {
        var i;
        if (result == false)
        {
            this.el.unmask();
            return;
        }
        if ((!data) || (!data.dsms))
        {
            Ext.Msg.alert(dSA._("Error"), dSA._("Could not load settings"));
            return;
        }

        dSA.functions.LoaddSMUpstreamFrequencies(
                this.LoadFrequenciesCallback.createDelegate(this,
                    [ data.dsms ], true));

    },
    LoadFrequenciesCallback: function(result, data1, data2)
    {
        var i, j;
        if (result == false)
        {
            this.el.unmask();
            return;
        }
        if (!data1 || !data2)
        {
            Ext.Msg.alert(dSA._("Error"), dSA._("Could not load settings"));
            return;
        }

        for (i = 0; i < data1.length; i++)
        {
            for (j = 0; j < data2.length; j++)
            {
                if (data1[i].dsuid == data2[j].dsuid)
                {
                    data1[i].name = data2[j].name;
                    continue;
                }
            }
        }

        this.dsmgrid.getStore().loadData(data1, false);
    },
    FreqSaveCallback: function(result, frequency, records)
    {
        if (records.length > 0)
        {
            var dsm = records.shift();
            dSA.functions.SavedSMUpstreamFrequency(dsm.get('dsuid'), frequency,
                this.FreqSaveCallback.createDelegate(this,
                                                [ frequency, records ], true));
        }
        else if (records.length === 0)
        {
            this.el.mask(dSA._("Loading") + '...',
                                 'x-mask-loading');
            dSA.functions.LoadStructure(
                this.LoadStructureCallback.createDelegate(this));
        }
    }
});

dSA.actions.dsmfreqsettings.ShowdSMFrequencySettingsDialog = function(maskEl)
{
    var freqform = new dSA.actions.dsmfreqsettings.FrequencySettings();

    var savebutton = new Ext.Button({
        text: dSA._("Apply"),
        handler: function()
        {
            freqform.fireEvent('savepressed');
        }
    });

    var win = new dSA.dialogs.DefaultDialogWindow(
    {
        title: dSA._("dSM Upstream Frequency Settings"),
        maskElement: maskEl,
        noinfo: true,
        extraItems: [ freqform ],
        buttons: [ savebutton ]
    });
    win.on('datasaved',
        function()
        {
            if (maskEl) 
            {
                maskEl.unmask();
            }
            win.close();
        });

    win.show();
};
